from setuptools import setup

setup(name='hundred_hammers',
      version='0.1.1',
      description='Batch execution of ML models to a dataset',
      url='http://github.com/vgarciasc/hundred_hammers',
      author='Vinicius Garcia',
      author_email='vinizinho@vinizinho.net',
      license='MIT',
      packages=['hundred_hammers'],
      zip_safe=False)